// AUTO-GENERATED BY UpdateGenVersion.bat 744352a150188b462764cb9b43c1974bc3ea8a5a
const char GenVersion_Version[] = "0.4.4-dev";
const char GenVersion_Commit[] = "744352a150188b462764cb9b43c1974bc3ea8a5a";
